/*
 *  This file is part of ScaffoldSelection, a method for the
 *  identification of potential scaffolds
 *  for the attachment of catalytic residues in a defined geometry.
 *
 *  ScaffoldSelection is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  ScaffoldSelection is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with ScaffoldSelection.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MOTIFRECONSTRUCTION_H_
#define MOTIFRECONSTRUCTION_H_

#include <BALL/COMMON/limits.h>
#include <BALL/STRUCTURE/defaultProcessors.h>
#include <BALL/STRUCTURE/residueChecker.h>
#include <BALL/DATATYPE/hashGrid.h>
#include <BALL/COMMON/limits.h>
#include "motif.h"
#include "motifHit.h"

class MotifConstruction
{

private:

	// the PDB system that will accomodate the motif
	System system;

	// the motif definition
	Motif motif;

	// the name of the PDB file
	String pdb_file;

	// the name of the PDB output file
	String output_file;

	// sequence positions of the anchor residues that will be replaced by the motif residues
	vector<String> sequence_positions;

	// the chains of the anchor residues in the same order as their positions
	vector<String> chains;

	// pointers to the anchor residues
	vector<Residue*> anchor_residues;

	// the indices of the rotamers in the motif's rotamer sets that have the minimal geometry penalty score
	// and are used for the motif construction
	vector<unsigned int> graft_rotamer_set;

	vector<Vector3> ideal_interaction_positions;

	MotifHit motif_hit;

	void determine_interaction_centers ();

	void construct_ligand_position ();

	void graft_motif_on_scaffold ();

	void read_parameter_file (String file_name);


public:

	/*
	 * Constructor
	 * reads in the PDB structure, and maps the motif
	 * on the residues at the provided positions in the motif
	 */
	MotifConstruction(Motif& motif_ref, String parameter_file);

	virtual ~MotifConstruction();

	/*
	 * identifies the anchor residues for the motif hit and
	 * maps the motif residues onto these anchors.
	 * writes the altered structure into a PDB file
	 */
	void construct_motif();

};

#endif /*MOTIFRECONSTRUCTION_H_*/
