#####################
# external dependency
#####################

# this is a list of all external packages - each entry has space separated: name of package, version
SET(
  SL_EXTERNAL_PACKAGES
  "ball 1.4.2"
  "boost 1.55.0"
  "gsl 1.16"
  "lpsolve 5.5"
  "python 2.7.6"
  "qt 4.8.5"
  "xdrportable 4.9.1"
)

# externals to consider from this distribution
SET( SL_EXTERNALS ball boost gsl lpsolve python qt xdrportable)

# define the external libs folder for os and architecture
IF( ${CMAKE_SYSTEM_NAME} STREQUAL "Linux")
  SET( SL_EXTERNAL_LIBRARY_BASE extern/Linux2/${CMAKE_SYSTEM_PROCESSOR}) #x86 and x86_64
  SET( SL_EXTERN_DESTINATION "lib")
ELSEIF( ${CMAKE_SYSTEM_NAME} STREQUAL "Darwin")
  SET( SL_EXTERNAL_LIBRARY_BASE extern/MacOS/${CMAKE_SYSTEM_VERSION}) #i386 and x86_64
  SET( SL_EXTERN_DESTINATION "lib")
ELSEIF( ${CMAKE_SYSTEM_NAME} STREQUAL "Windows")
  SET( SL_EXTERNAL_LIBRARY_BASE extern/windows/${CMAKE_SYSTEM_PROCESSOR}) #i686 and x86_64
  SET( SL_EXTERN_DESTINATION "bin")
ENDIF()

# add include and library path for external packages and find package
FOREACH( EXTERNAL ${SL_EXTERNAL_PACKAGES})
  SEPARATE_ARGUMENTS( EXTERNAL)
  LIST( GET EXTERNAL 0 CURRENT_PACKAGE) 
  LIST( GET EXTERNAL 1 CURRENT_VERSION)
  SET( CMAKE_INCLUDE_PATH ${CMAKE_CURRENT_SOURCE_DIR}/extern/noarch/${CURRENT_PACKAGE}/${CURRENT_VERSION}/include ${CMAKE_INCLUDE_PATH})
  SET( CMAKE_LIBRARY_PATH ${CMAKE_CURRENT_SOURCE_DIR}/${SL_EXTERNAL_LIBRARY_BASE}/${CURRENT_PACKAGE}/${CURRENT_VERSION}/${SL_EXTERN_DESTINATION} ${CMAKE_LIBRARY_PATH})
  # if package is supposed to be used
  IF( SL_EXTERNALS STREQUAL OFF OR SL_EXTERNALS MATCHES ${CURRENT_PACKAGE})
    MESSAGE( STATUS "package is considered: " ${CURRENT_PACKAGE} " " ${CURRENT_VERSION})
    FIND_PACKAGE( ${CURRENT_PACKAGE} ${CURRENT_VERSION})
    # if package was found
    IF( ${CURRENT_PACKAGE}_FOUND)
      MESSAGE( STATUS "found " ${CURRENT_PACKAGE} " " ${CURRENT_VERSION})
      INCLUDE_DIRECTORIES( ${${CURRENT_PACKAGE}_INCLUDE_DIRS})
      LIST( APPEND EXTERN_LINK_LIBRARIES ${${CURRENT_PACKAGE}_LIBRARIES})
      LIST( APPEND EXTERN_SHARES ${${CURRENT_PACKAGE}_SHARE})

      # find license file
      FIND_FILE( ${CURRENT_PACKAGE}_LICENSE NAMES "LICENSE.txt" PATHS ${CMAKE_CURRENT_SOURCE_DIR}/extern/noarch/${CURRENT_PACKAGE} DOC "license file for package ${CURRENT_PACKAGE}" NO_DEFAULT_PATH)
      IF( ${CURRENT_PACKAGE}_LICENSE)
        INSTALL(
          FILES ${${CURRENT_PACKAGE}_LICENSE}
          DESTINATION ${SL_EXTERN_DESTINATION}
          RENAME ${CURRENT_PACKAGE}License.txt
          COMPONENT ${CURRENT_PACKAGE}_LICENSE
        )
        CPACK_ADD_COMPONENT(
          ${CURRENT_PACKAGE}_LICENSE
          DISPLAY_NAME "${CURRENT_PACKAGE} ${CURRENT_VERSION} license information"
          DESCRIPTION "external library: ${CURRENT_PACKAGE} with version: ${CURRENT_VERSION} - license information"
          GROUP SlExtern
        )
      ENDIF()
    
      # check if runtime libraries need to be installed
      IF( ${CURRENT_PACKAGE}_RUNTIME_LIBRARY)
        SET( ${CURRENT_PACKAGE}_RUNTIME_LIBRARIES ${${CURRENT_PACKAGE}_RUNTIME_LIBRARY})
      ENDIF()
      IF( ${CURRENT_PACKAGE}_RUNTIME_LIBRARY_RENAME)
        SET( ${CURRENT_PACKAGE}_RUNTIME_LIBRARIES_RENAME ${${CURRENT_PACKAGE}_RUNTIME_LIBRARY_RENAME})
      ENDIF()

      # check if multiple runtime libraries need to be installed
      IF( ${CURRENT_PACKAGE}_RUNTIME_LIBRARIES)
        LIST( LENGTH ${CURRENT_PACKAGE}_RUNTIME_LIBRARIES        NR_RUNTIME_LIBS)
        LIST( LENGTH ${CURRENT_PACKAGE}_RUNTIME_LIBRARIES_RENAME NR_RUNTIME_LIBS_RENAME)
        MATH(EXPR NR_ITERATIONS "${NR_RUNTIME_LIBS} - 1")

        CPACK_ADD_COMPONENT(
          ${CURRENT_PACKAGE}
          DISPLAY_NAME "${CURRENT_PACKAGE} ${CURRENT_VERSION}"
          DESCRIPTION "external library: ${CURRENT_PACKAGE} with version: ${CURRENT_VERSION} - will be necessary, if not installed on the system"
          GROUP SlExtern
        )
        FOREACH( I RANGE ${NR_ITERATIONS})
          LIST( GET ${CURRENT_PACKAGE}_RUNTIME_LIBRARIES ${I} CURRENT_RUNTIME_LIBRARY)
          IF( ${NR_RUNTIME_LIBS} EQUAL ${NR_RUNTIME_LIBS_RENAME})
            LIST( GET ${CURRENT_PACKAGE}_RUNTIME_LIBRARIES_RENAME ${I} CURRENT_RUNTIME_LIBRARY_RENAME)
            MESSAGE( STATUS "add runtime library to install: " ${CURRENT_RUNTIME_LIBRARY} " and so name: " ${CURRENT_RUNTIME_LIBRARY_RENAME})
            INSTALL(
              FILES ${CURRENT_RUNTIME_LIBRARY}
              DESTINATION ${SL_EXTERN_DESTINATION}
              RENAME ${CURRENT_RUNTIME_LIBRARY_RENAME}
              COMPONENT ${CURRENT_PACKAGE}
            )
          ELSE( ${NR_RUNTIME_LIBS} EQUAL ${NR_RUNTIME_LIBS_RENAME})
            MESSAGE( STATUS "add runtime library to install: " ${CURRENT_RUNTIME_LIBRARY})
            INSTALL(
              FILES ${CURRENT_RUNTIME_LIBRARY}
              DESTINATION ${SL_EXTERN_DESTINATION}
              COMPONENT ${CURRENT_PACKAGE}
            )
          ENDIF( ${NR_RUNTIME_LIBS} EQUAL ${NR_RUNTIME_LIBS_RENAME})
        ENDFOREACH( I RANGE ${NR_ITERATIONS})
      ENDIF( ${CURRENT_PACKAGE}_RUNTIME_LIBRARIES)
    ENDIF()
  ENDIF()
ENDFOREACH()

# install the external shares
INSTALL( 
  DIRECTORY ${EXTERN_SHARES} 
  DESTINATION .
  COMPONENT Share
)
CPACK_ADD_COMPONENT(
  Share
  DISPLAY_NAME "shared resources"
  DESCRIPTION "shared resources like databanks and scripts"
  GROUP SlExtern
)
