# runtime for gnu cxx compiler
IF( CMAKE_COMPILER_IS_GNUCXX)
  # add libgcc_s_sjlj-1.dll and libstdc++-6.dll to installer
  EXECUTE_PROCESS( COMMAND ${CMAKE_CXX_COMPILER} -print-file-name= OUTPUT_VARIABLE CXX_COMPILER_ROOT)
  STRING( REGEX REPLACE "[\n\r\t]+" "" CXX_COMPILER_ROOT ${CXX_COMPILER_ROOT})
  SET( CXX_COMPILER_ROOT ${CXX_COMPILER_ROOT}/../../../../)
  GET_FILENAME_COMPONENT( CXX_COMPILER_ROOT ${CXX_COMPILER_ROOT} ABSOLUTE)
  MESSAGE( STATUS "compiler root: " ${CXX_COMPILER_ROOT})
  
  # runtime library search paths
  SET( RUNTIME_LIBRARY_SEARCH_PATHS_SUFFIXES bin)
  IF( ${CMAKE_SYSTEM_PROCESSOR} STREQUAL "x86_64")
    LIST( APPEND RUNTIME_LIBRARY_SEARCH_PATHS_SUFFIXES lib64)
  ELSEIF( ${CMAKE_SYSTEM_PROCESSOR} STREQUAL "x86")
    LIST( APPEND RUNTIME_LIBRARY_SEARCH_PATHS_SUFFIXES lib)
  ENDIF()

  # runtime library file names
  IF( MINGW)
    LIST( APPEND RUNTIME_LIBRARY_SEARCH_PATHS_SUFFIXES bin)
    SET( RUNTIME_FILES libstdc++-6.dll libgcc_s_sjlj-1.dll)
  ELSEIF( UNIX)
    SET( RUNTIME_FILES libstdc++.so.6 libgcc_s.so.1)
  ENDIF()
  
  FOREACH( FILE ${RUNTIME_FILES})
    FIND_FILE( INSTALL_RUNTIME_FILE ${FILE} PATHS ${CXX_COMPILER_ROOT} PATH_SUFFIXES ${RUNTIME_LIBRARY_SEARCH_PATHS_SUFFIXES} NO_DEFAULT_PATH)
    # using readlink here is generally bad because it is unix-dependent; GET_FILENAME_COMPONENT should be used, but
    # unfortunately contains a bug (which is likely fixed in newer versions of cmake, see http://www.cmake.org/pipermail/cmake/2009-January/026694.html)
    EXECUTE_PROCESS( COMMAND readlink -f ${INSTALL_RUNTIME_FILE}   OUTPUT_VARIABLE INSTALL_RUNTIME_FILE)
    # remove any new line characters that are present
    STRING( REGEX REPLACE "\n" "" INSTALL_RUNTIME_FILE "${INSTALL_RUNTIME_FILE}")
    LIST( APPEND INSTALL_RUNTIME_FILES ${INSTALL_RUNTIME_FILE})
    UNSET( INSTALL_RUNTIME_FILE CACHE)
    UNSET( INSTALL_RUNTIME_FILE)
  ENDFOREACH()
  
  # create links to the external libraries for wine
  IF( WINE_LIBRARY_PATH)
    FOREACH( FILE ${INSTALL_RUNTIME_FILES})
      GET_FILENAME_COMPONENT( LIBRARY_FILE_NAME ${FILE} NAME)
      EXECUTE_PROCESS( COMMAND ln -s -f "${FILE}" "${WINE_LIBRARY_PATH}/${LIBRARY_FILE_NAME}")
    ENDFOREACH()
  ENDIF()
  
  IF( INSTALL_RUNTIME_FILES)
    MESSAGE( STATUS "found runtime files to install: ${INSTALL_RUNTIME_FILES}")
    IF( MINGW)
      INSTALL(
        FILES ${INSTALL_RUNTIME_FILES}
        DESTINATION ${SL_EXTERN_DESTINATION}
        COMPONENT StdRuntimeLibraries
      )
    ELSEIF( UNIX)
      FOREACH( INSTALL_RUNTIME_FILE ${INSTALL_RUNTIME_FILES})
        MACRO_GET_SO_NAME( ${INSTALL_RUNTIME_FILE})
        INSTALL(
          FILES ${INSTALL_RUNTIME_FILE}
          DESTINATION ${SL_EXTERN_DESTINATION}
          RENAME ${SO_FILE_NAME}
          COMPONENT StdRuntimeLibraries
        )
        UNSET( SO_FILE_NAME)
      ENDFOREACH()
    ENDIF()
    CPACK_ADD_COMPONENT(
      StdRuntimeLibraries
      DISPLAY_NAME "std runtime libraries"
      DESCRIPTION "runtime libraries required due to linking by compiler"
      GROUP BclExtern
    )
  ENDIF()
ENDIF()
