/*
 *  This file is part of ScaffoldSelection, a method for the
 *  identification of potential scaffolds
 *  for the attachment of catalytic residues in a defined geometry.
 *
 *  ScaffoldSelection is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  ScaffoldSelection is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with ScaffoldSelection.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <BALL/DATATYPE/string.h>
#include <BALL/SYSTEM/timer.h>
#include <dirent.h>

#include "scaffoldSearch.h"

using namespace std;
using namespace BALL;


/**
 * Starts a ScaffoldSelection run
 * needs as input a parameter file and a motif definition file
 */
int main(int argc, char** argv)
{
	// print usage
	if (argc < 3)
	{
		cout << "usage:\n " << argv[0] << " <parameter ini file>  "
		        "<motif definition ini file>" << endl;
		return 1;
	}

	try
	{
		bool verbose = false;
		bool time_is_money = false;
		if (argc > 3)
		{
			if (String(argv[3]) == String("-time")) time_is_money = true;
			else if (String(argv[3]) == String("-verbose")) verbose = true;
			else throw "usage:\n " + String(argv[0]) + " <parameter file>"
			           "  <motif definition file> <output file>" ;
		}
		if (argc == 5)
		{
			if (String(argv[4]) == String("-time")) time_is_money = true;
			else if (String(argv[4]) == String("-verbose")) verbose = true;
			else throw "usage:\n " + String(argv[0]) + " <parameter file>"
			           "  <motif definition file> <output file>" ;
		}

		// initialize the parameters
		Parameters parameters(argv[1], verbose, time_is_money);

		// create the motif object
		Motif motif(argv[2]);
		cout << "Motif preparation finished." << endl;

		// create the scaffold search object
		ScaffoldSearch scaffold_search( parameters, motif);


		// start a search for mmotif hits in all PDB structures
		scaffold_search.start_scaffold_search();

	}
	catch (String &message)
	{
		cerr << message << endl;
	}
	catch (Exception::GeneralException &e)
	{
		cerr << "A " << e.getName() << " Exception occured in file \""
		     << e.getFile() << "\" line " << e.getLine()
		     << ":\n" << e.getMessage() << endl;
	}
	return 0;
}
